@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 if [ "x$1" = "x" ]; then
 	APP=org.kde.amarok
@@ -12,10 +12,15 @@ else
 	FILTER=$2
 fi; 
 
-length=`qdbus $APP /TrackList GetLength`
-for (( i=0; i<$length; i++ ))
+LENGTH=`qdbus $APP /TrackList GetLength`
+
+I=0
+
+while [ $I -lt $LENGTH ]
 do
    T=`qdbus $APP /TrackList GetMetadata $i |grep $FILTER |cut -f 2 -d ":"|tr -s "," " "`
    ni=`expr $i + 1`  
    echo "$ni.$T,"
+   
+   I=`expr $I + 1`
 done
