@@ -29,8 +29,10 @@
 #define TYPE_BOOL 	4
 #define TYPE_STRING 	5
 
+#include "parse.h"
+
 int  dbusInit();
 void dbusFinish();
-int  dbusCommand(char* cmdString);
+int  dbusCommand(int subtype, const char* descr, const char* cmdString, cmdParams* params);
 
 #endif
