@@ -22,9 +22,13 @@
 #ifndef _ATSEND_H_
 #define _ATSEND_H_
 
-int sendSeq 	(const char *string);
-int sendCMER	(int onOff);
-int getClip 	(char *string);
-void sendSyncCommand(const char *bytes, char *answer, int size, char *expect);
+#include "peer.h"
+
+int setupAtConnection(ConnectInfo* p, int fd);
+int atRead           (ConnectInfo* conn, char* buffer, int max);
+
+int sendSeq 	(int fd, const char *string);
+int sendCMER	(int fd, int onOff);
+int getClip 	(int fd, char *string);
 
 #endif
