@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -266,18 +266,18 @@ int uxFD(ConnectInfo* connInfo)
     return port_fd;
 }
 
-int openUnixSocket(ConnectInfo* connInfo)
+int openUxSocket(ConnectInfo* connInfo)
 {
     DEBUG2("[DS]: Unix socket client mode. Use port %s", connInfo->portStr->str);
     if (unix_open_port(connInfo->portStr->str) < 0) {
     	printf("ERROR: open inputlircd socket\n");
     	return EXIT_NOK;
     }
-    connInfo->connected = BOOL_YES;
+    connInfo->state = PEER_CONNECTED;
     return EXIT_OK;
 }
 
-void closeUnixSocket(ConnectInfo* connInfo, int final)
+void closeUxSocket(ConnectInfo* connInfo, int final)
 {
     int retval = 0;
     //printf("INFO: Close port\n");
@@ -289,7 +289,7 @@ void closeUnixSocket(ConnectInfo* connInfo, int final)
 
     retval = close(port_fd);
     port_fd = -1;
-    connInfo->connected = BOOL_NO;
+    connInfo->state = PEER_DISCONNECTED;
     
     if (retval < 0) {
         logger(L_ERR,"[DS]: Error on closing AF_UNIX socket\n");
