@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,8 +36,8 @@ int  bt_put_command(int portfd,
                     char* expect);
 
 int uxFD (ConnectInfo* connInfo);
-int  openUnixSocket (ConnectInfo* connInfo);
-void closeUnixSocket(ConnectInfo* connInfo, int final);
+int  openUxSocket (ConnectInfo* connInfo);
+void closeUxSocket(ConnectInfo* connInfo, int final);
 //int unix_read_port(char* buf, int l);
 
 int readSocketConn(char* buffer, int len);
