@@ -371,7 +371,13 @@ int handleHook(int hook)
     DEBUG2("handleHook() >%d<",hook);
 
     char hookFile[MAXLEN];
-    strcpy(hookFile, getenv("HOME"));
+    char *t = getenv("HOME");
+    if (t) {
+        strcpy(hookFile, t);
+    } else {
+        strcpy(hookFile, ".");
+    }
+    
     strcat(hookFile, "/.anyRemote/");
     strcat(hookFile, _hookNames[hook]);
 
