@@ -39,9 +39,9 @@
 void	setToFile	(void);
 int  	handleCmdByKey  (type_key* k, cmdParams *p);
 int     handleCmdByKeyEx(type_key* k, cmdParams* p, int sendToMain);
-int 	processCommands (cmdItem* cmdList, cmdParams *p);
 int 	execDynamically (char *command);
 void	flushOldConf	(void);
 int	handleHook      (int hook);
+char*   executeCommandPipe(const char* exec, size_t* sz);
 
 #endif
