@@ -23,15 +23,14 @@
 #define _CONF_H_
 
 #include "parse.h"
+#include "list.h"
+#include "mode.h"
 
-char* getFirstAlarm     (char **cmds);
-char* getNextAlarm      (char **cmds);
 char* getAT_CMER     	(int what);
 char* getCfgDir         (void);
 char* getConfigDir      (void);
 char* getCharset        (void);
 char* getDevice         (void);
-char* getMode           (void);
 char* getToMainMenu     (void);
 char* getServiceName    (void);
 int   autoConnect       (void);
@@ -39,26 +38,38 @@ int   getAutoRepeat     (void);
 int   getBaudrate       (void);
 int   getDebug          (void);
 int   getFrontEnd       (void);
-int   getLog            (void);
 int   getModel          (void);
 int   getRetrySecs  	(void);
+int   getWaitTime       (void);
 int   getUseScreen      (void);
-int   getHttp           (void);
 int   getIViewer        (void);
-int   usePassword       (void);
-void  addAlarm		(char *file, char *macro);
 void  freeCfg           (void);
-void  freeCfgEx         (mode* pmodes, type_alarm* palarms);
-void  freeCmds          (cmdItem* item);
+void  freeCmds          (SingleList* commands);
 void  printConf         (void);
-void  printKeys		(void);
+void  printKeys         (type_key* k);
 void  setModel          (char *m);
 void  setInitDone       (void);
 
-void setRepeatNow       (type_key *k);
+boolean_t getLog        (void);
+boolean_t getBemused    (void);
+boolean_t getIViewer    (void);
+
+void  setLog            (const char* value);
+void  setBemused        (const char* value);
+void  setIViewer        (const char* value);
+void  setWaitTime       (const char* value);
+
+void  setRepeatNow       (type_key *k);
 type_key* repeatNow     ();
 
 type_key* findItem	(const char *key, int *flag, cmdParams *param);
-cmdItem* getCommand	(type_key* item);
+type_key* findExact     (mode *mode, const char *key);
+SingleList* getCommand	(type_key* item);
+
+void  setBtAddress (char* a);
+char* getBtAddress ();
+void  freeBtAddress(void);
+
+boolean_t boolValue(const char* value);
 
 #endif
