@@ -22,23 +22,24 @@
 #ifndef _EXECUTOR_H_
 #define _EXECUTOR_H_
 
-#include <glib.h>
+#include "lib_wrapper.h"
 
-#define EM_KEY		1
-#define EM_STRING	2
-#define EM_ITEM		3
-#define EM_EVENT	4
-#define EM_ALARM	5
-
-#define EM_AS_IS 	99
+enum ExecMsgType {
+    EM_KEY = 0,
+    EM_STRING,
+    EM_EVENT,
+    EM_ALARM,
+    EM_AS_IS
+};
 
 typedef struct em {
-    int        type;
-    void*      value;
+    int   type;
+    void* value;
 } eMessage;
 
-gpointer executorRoutine	(gpointer thread);
+pointer_t executorRoutine	(pointer_t thread);
 void  	 sendToExecutor  	(eMessage *buf);
+void  	 freeEMessage  	        (void *buf);
 void  	 sendEventToExecutor	(int event);
 void  	 sendToMainMenu		(void);
 
