@@ -32,12 +32,12 @@ enum arCookieStatus {
 
 void      initHtmlGenerator();
 
-string_t* renderCtrlHTMLForm(int port);  
-string_t* renderTextHTMLForm(int port);   
-string_t* renderListHTMLForm(int port);
-string_t* renderWmanHTMLForm(int port);
-string_t* renderEditHTMLForm(int port);
-string_t* renderPassHTMLForm(int port);
+string_t* renderCtrlHTMLForm(string_t* ip, int port);  
+string_t* renderTextHTMLForm(string_t* ip, int port);   
+string_t* renderListHTMLForm(string_t* ip, int port);
+string_t* renderWmanHTMLForm(string_t* ip, int port);
+string_t* renderEditHTMLForm(string_t* ip, int port);
+string_t* renderPassHTMLForm(string_t* ip, int port);
 
 int  htmlScreenWidth ();
 int  htmlScreenHeight();
