@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2011-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2011-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,11 +30,10 @@
 #include "sys_util.h"
 #include "state.h"
 #include "gen_xml.h"
+#include "mutex.h"
 
 // not to include pr_web.h
-// TODO: rewrite
-extern void addLink(string_t* addTo, int port, boolean_t trailSlash);
-extern char* confDir;
+extern void addLink(string_t* ip, string_t* addTo, int port, boolean_t trailSlash);
 
 #define CMXML_MAX_STRING_SIZE 32
 
@@ -143,6 +142,8 @@ void parseScreenDef(char* buffer)
     _xmlScreenW = sW;
     _xmlScreenH = sH;
     _xmlScreenG = sG;
+    
+    free(scrDef);
 }
 
 static void renderConvertCmd(string_t* cmd)
@@ -166,7 +167,7 @@ static void renderConvertCmd(string_t* cmd)
     }
 }
 
-static void addXMLFormMenu(int formId, string_t* page, int p)
+static void addXMLFormMenu(int formId, string_t* ip, string_t* page, int p)
 {
     SingleList* list = userMenu();
 
@@ -190,7 +191,7 @@ static void addXMLFormMenu(int formId, string_t* page, int p)
                 if (longMenu && position == XML_SOFTKEY_NUM) {
 
                     stringAppend(page, "  <SoftKeyItem>\n    <Name>>></Name>\n    <URL>");
-                    addLink(page, p, BOOL_YES);
+                    addLink(ip, page, p, BOOL_YES);
                     stringAppend(page, XML_LONG_MENU);
 
                     char buf[8];
@@ -213,7 +214,7 @@ static void addXMLFormMenu(int formId, string_t* page, int p)
                     stringAppend(page, ((string_t*) list->data)->str);
                     stringAppend(page, "</Name>\n    <URL>");
 
-                    addLink(page, p, BOOL_YES);
+                    addLink(ip, page, p, BOOL_YES);
 
                     char buf[8];
                     sprintf(buf,"%d", position);
@@ -252,7 +253,7 @@ static void addXMLFormMenu(int formId, string_t* page, int p)
     		stringAppend(page, ((string_t*) list->data)->str);
     		stringAppend(page, "</Name>\n    <URL>");
 
-    		addLink(page, port, BOOL_YES);
+    		addLink(ip, page, port, BOOL_YES);
 
     		char buf[8];
     		sprintf(buf,"%d", position);
@@ -299,21 +300,21 @@ string_t* renderXMLFormHead(int form, const char* caption)
     stringAppend(head, "  <Prompt>");
     
     if (form == CF) {
-	const char* title = cfTitle();
-	int sz = (title ? strlen(title) : -1);
-	if (sz > 0) {
-	    if (sz > CMXML_MAX_STRING_SIZE) { 
+	    const char* title = cfTitle();
+	    int sz = (title ? strlen(title) : -1);
+	    if (sz > 0) {
+	        if (sz > CMXML_MAX_STRING_SIZE) { 
                 stringAppendLen(head, cfTitle(), CMXML_MAX_STRING_SIZE);
-	    } else {
+	        } else {
                 stringAppend(head, title);
+	        }
 	    }
-	}
     } else {
         if (strlen(caption) > CMXML_MAX_STRING_SIZE) {
             stringAppendLen(head, caption, CMXML_MAX_STRING_SIZE);
-	} else {
-	    stringAppend(head, caption);
-	}
+	    } else {
+	        stringAppend(head, caption);
+	    }
     }
     stringAppend(head, "</Prompt>\n");
 
@@ -322,7 +323,7 @@ string_t* renderXMLFormHead(int form, const char* caption)
 }
 
 // Send menu as list (CMXML)
-string_t* sendXMLMenu(int form, int p, int idx)
+string_t* sendXMLMenu(int form, string_t* ip, int p, int idx)
 {
     string_t* page = renderXMLFormHead(LI,"Menu");  // menu-as-list
 
@@ -345,7 +346,7 @@ string_t* sendXMLMenu(int form, int p, int idx)
                 }
                 stringAppend(page, "</Name>\n   <URL>");
 
-                addLink(page, p, BOOL_YES);
+                addLink(ip, page, p, BOOL_YES);
 
                 char pos[8];
                 sprintf(pos, "%d",position);
@@ -376,15 +377,21 @@ string_t* sendXMLMenu(int form, int p, int idx)
 
 static void renderXMLLayout()
 {
+    mutexLock(M_STATE);
+    
     const char* cv = cfCover();
+    const char* nc = cfNamedCover();
    
     INFO2("[WS]: renderXMLLayout (%d,%d) pix, %s, %s", _xmlScreenW, _xmlScreenH,
           (_xmlScreenG ? "Grayscale" : "Color"),
-          (cv ? "with cover" : "no cover"));
+          ((cv || nc) ? "with cover" : "no cover"));
 
     // Use ImageMagic to create picture to show
 
+    char* confDir = getCfgDir();
     string_t* iconpath = stringNew(confDir ? confDir : ".");
+    free(confDir);
+    
     stringAppend(iconpath,"/Icons/32/");
 
     string_t* cmd = NULL;
@@ -404,7 +411,7 @@ static void renderXMLLayout()
 
         cmd = stringNew("montage -background transparent -tile ");
 
-        char num[16];
+        char num[32];
         sprintf(num,"%dx1 -geometry 32x32 ", iNum);
         stringAppend(cmd, num);
 
@@ -415,19 +422,27 @@ static void renderXMLLayout()
         int i;
         for (i=0; i<iNum; i++) {
 	
-	    const char* icon = cfIcon(i);
-	    if (icon) {
-        	stringAppend(cmd, iconpath->str);
-        	stringAppend(cmd, icon);
-        	stringAppend(cmd, ".png ");
-	    }
+	        const char* icon = cfIcon(i);
+	        if (icon) {
+        	    stringAppend(cmd, iconpath->str);
+        	    stringAppend(cmd, icon);
+        	    stringAppend(cmd, ".png ");
+	        }
         }
 
-        if (cv) {
-            stringAppend(cmd, " $HOME/.anyRemote/layout_xml_raw0.png; convert -gravity Center -background transparent ");
-            stringAppend(cmd, cv);
+        if (nc || cv) {
+            
+           stringAppend(cmd, " $HOME/.anyRemote/layout_xml_raw0.png; convert -gravity Center -background transparent ");
+           if (nc) {
+                 string_t* file = findNamedCover(nc);
+                 if (file) {
+                     stringAppend(cmd, file->str);
+                     stringFree(file, BOOL_YES);
+                 }
+            } else {  // cv
+                 stringAppend(cmd, cv);
+            }
             stringAppend(cmd, " $HOME/.anyRemote/layout_xml_raw0.png -append $HOME/.anyRemote/layout_xml_raw.png; ");
-
         } else {
             stringAppend(cmd, " $HOME/.anyRemote/layout_xml_raw.png; ");
         }
@@ -443,15 +458,16 @@ static void renderXMLLayout()
 
         int i;
         for (i=0; i<ICON_NUM; i++) {
-	    const char* icon = cfIcon(i);
-	    if (icon) {
+	        const char* icon = cfIcon(i);
+	        if (icon) {
                 stringAppend(cmd, iconpath->str);
-        	stringAppend(cmd, icon);
-        	stringAppend(cmd, ".png ");
-	    }
+        	    stringAppend(cmd, icon);
+        	    stringAppend(cmd, ".png ");
+	        }
         }
         stringAppend(cmd, " $HOME/.anyRemote/layout_xml_raw.png; convert $HOME/.anyRemote/layout_xml_raw.png ");
     }
+    mutexUnlock(M_STATE);
 
     renderConvertCmd(cmd);
 
@@ -499,7 +515,12 @@ void renderXMLImage()
     // Use ImageMagic to create picture to show
 
     string_t* cmd = stringNew("convert ");
+    
+    mutexLock(M_STATE);
+    
     stringAppend(cmd, wfImage());
+    
+    mutexUnlock(M_STATE);
 
     renderConvertCmd(cmd);
 
@@ -515,15 +536,17 @@ void renderXMLImage()
     stringFree(cmd, BOOL_YES);
 }
 
-string_t* renderCtrlXMLForm(int port)
+string_t* renderCtrlXMLForm(string_t* ip, int port)
 {
     INFO2("[WS]: renderCtrlXMLForm");
 
     if (!_xmlLayoutOk) {
         renderXMLLayout();
     }
-
+    
+    mutexLock(M_STATE);
     string_t* page = renderXMLFormHead(CF, cfCaption());
+    mutexUnlock(M_STATE);
     
     stringAppend(page, "  <LocationX>");
     
@@ -542,11 +565,14 @@ string_t* renderCtrlXMLForm(int port)
 
     stringAppend(page, "  <URL>");
 
-    addLink(page, port, BOOL_YES);
+    addLink(ip, page, port, BOOL_YES);
 
     stringAppend(page, "xml_layout</URL>\n");
 
     int i = 0;
+    
+    mutexLock(M_STATE);
+    
     while (i < ICON_NUM) {
 
         const char* icon = cfIcon(i);
@@ -559,7 +585,7 @@ string_t* renderCtrlXMLForm(int port)
         stringAppend(page, icon);
         stringAppend(page, "</Name>\n   <URL>");
 
-        addLink(page, port, BOOL_YES);
+        addLink(ip, page, port, BOOL_YES);
 
         stringAppend(page, XML_BUTTON_PRESS);
 
@@ -570,8 +596,9 @@ string_t* renderCtrlXMLForm(int port)
 
         i++;
     }
+    mutexUnlock(M_STATE);
 
-    addXMLFormMenu(CF, page, port);
+    addXMLFormMenu(CF, ip, page, port);
 
     stringAppend(page, XML_CFORM99);
     
@@ -579,25 +606,32 @@ string_t* renderCtrlXMLForm(int port)
     return page;
 }
 
-string_t* renderTextXMLForm(int port)
+string_t* renderTextXMLForm(string_t* ip, int port)
 {
+    mutexLock(M_STATE);
     string_t* page = renderXMLFormHead(TX,tfCaption());
-
+     
     stringAppend(page, "  <Text>");
+    
     stringAppend(page, tfText());
+    
+    mutexUnlock(M_STATE);
+    
     stringAppend(page, "&#10;</Text>");
 
-    addXMLFormMenu(TX, page, port);
+    addXMLFormMenu(TX, ip, page, port);
 
     stringAppend(page, XML_TEXT99);
     
     return page;
 }
 
-string_t* renderListXMLForm(int port)
+string_t* renderListXMLForm(string_t* ip, int port)
 {
+    mutexLock(M_STATE);
+    
     string_t* page = renderXMLFormHead(LI,lfCaption());
-
+    
     SingleList* list = lfList();
     char num[16];
 
@@ -637,6 +671,8 @@ string_t* renderListXMLForm(int port)
         stringAppend(page, "-1");
         stringAppend(page, "</URL>\n  </MenuItem>\n");
     }
+   
+    mutexUnlock(M_STATE);
 
     SingleList* mlist = userMenu();
 
@@ -661,7 +697,7 @@ string_t* renderListXMLForm(int port)
                 }
                 stringAppend(page, "</Name>\n    <URL>");
 
-                addLink(page, port, BOOL_YES);
+                addLink(ip, page, port, BOOL_YES);
 
                 char buf[8];
                 sprintf(buf,"%d", position);
@@ -692,33 +728,35 @@ string_t* renderListXMLForm(int port)
     return page;
 }
 
-string_t* renderWmanXMLForm(int port)
+string_t* renderWmanXMLForm(string_t* ip, int port)
 {
     INFO2("[WS]: renderWmanXMLForm");
 
+    mutexLock(M_STATE);
     string_t* page = renderXMLFormHead(WM, cfCaption());
-
+    mutexUnlock(M_STATE);
+    
     stringAppend(page, "  <LocationX>0</LocationX>\n");
     stringAppend(page, "  <LocationY>0</LocationY>\n");
     stringAppend(page, "  <URL>");
 
-    addLink(page, port, BOOL_YES);
+    addLink(ip, page, port, BOOL_YES);
 
     stringAppend(page, "xml_image</URL>\n");
 
-    addXMLFormMenu(WM, page, port);
+    addXMLFormMenu(WM, ip, page, port);
 
     stringAppend(page, XML_WMAN99);
     
     return page;
 }
 
-static string_t* renderEditXML(const char* caption, const char* label, const char* value, int p)
+static string_t* renderEditXML(const char* caption, const char* label, const char* value, string_t* ip, int p)
 {
     string_t* page = renderXMLFormHead(EF, caption);
 
     stringAppend(page, "  <URL>");
-    addLink(page, p, BOOL_YES);
+    addLink(ip, page, p, BOOL_YES);
     stringAppend(page, ";");
 
     stringAppend(page, "</URL>\n  <InputItem>\n   <DisplayName>");
@@ -727,11 +765,11 @@ static string_t* renderEditXML(const char* caption, const char* label, const cha
     stringAppend(page, value);
     stringAppend(page, "</DefaultValue>\n   <InputFlags>A</InputFlags>\n  </InputItem>\n");
 
-    //addXMLFormMenu(EF, page, p);  -- add hardcoded Ok and Back menu items
+    //addXMLFormMenu(EF, ip, page, p);  -- add hardcoded Ok and Back menu items
 
     stringAppend(page, "  <SoftKeyItem>\n    <Name>Ok</Name>\n    <URL>SoftKey:Submit</URL>\n    <Position>1</Position>\n  </SoftKeyItem>\n");
     stringAppend(page, "  <SoftKeyItem>\n    <Name>Back</Name>\n    <URL>");
-    addLink(page, p, BOOL_YES);
+    addLink(ip, page, p, BOOL_YES);
     stringAppend(page, XML_EFIELD_CANCEL);
     stringAppend(page, "</URL>\n    <Position>2</Position>\n  </SoftKeyItem>\n");
     stringAppend(page, "  <SoftKeyItem>\n    <Name>&lt;&lt;</Name>\n    <URL>SoftKey:&lt;&lt;</URL>\n    <Position>3</Position>\n  </SoftKeyItem>\n");
@@ -741,14 +779,17 @@ static string_t* renderEditXML(const char* caption, const char* label, const cha
     return page;
 }
 
-string_t* renderEditXMLForm(int port)
+string_t* renderEditXMLForm(string_t* ip, int port)
 {
     INFO2("[WS]: renderXMLEditForm");
-    return renderEditXML(efCaption(), efLabel(), efText(), port);
+    mutexLock(M_STATE);
+    string_t* val = renderEditXML(efCaption(), efLabel(), efText(), ip, port);
+    mutexUnlock(M_STATE);
+    return val;
 }
 
-string_t* renderPassXMLForm(int port)
+string_t* renderPassXMLForm(string_t* ip, int port)
 {
     INFO2("[WS]: renderPassXMLForm");
-    return renderEditXML("Enter Password", "Enter Password", "", port);
+    return renderEditXML("Enter Password", "Enter Password", "", ip, port);
 }
