@@ -144,7 +144,11 @@ static void createDataDir()
 {
     char dd[542];
     char *t = getenv("HOME");
-    strcpy(dd, t);
+    if (t) {
+        strcpy(dd, t);
+    } else {
+        strcpy(dd, ".");
+    }
     strcat(dd, "/.anyRemote");
 
     DIR *d = opendir(dd);
