@@ -1557,7 +1557,8 @@ static string_t* loadLine(FILE *fp)
 static int loadKeys(FILE *fp)
 {
     string_t *tmptext;
-    char *tag, *value;
+    char *tag   = NULL;
+    char *value = NULL;
     
     mode* def   = getDefaultMode();  // will create it if absent
     mode* store = getCurrentMode();
@@ -1780,7 +1781,12 @@ int init_cfg(char *path)
     } else {
 
         char cfgfile[MAXLEN];
-        strcpy(cfgfile, getenv("HOME"));
+        char *t = getenv("HOME");
+        if (t) {
+            strcpy(cfgfile, t);
+        } else {
+            strcpy(cfgfile, ".");
+        }
         strcat(cfgfile, CFGFILE);
 
         printf("Search configuration file %s\n", cfgfile);
