@@ -124,6 +124,7 @@ enum CommandID {
 #define CMD_DBUS        "Dbus"
 
 #define SET_BG          "bg"
+#define SET_CAPTION     "caption"
 #define SET_PARAM       "parameter"
 #define SET_EFIELD      "editfield"
 #define SET_FG          "fg"
@@ -149,6 +150,7 @@ enum CommandID {
 
 enum CommandSetID {
     ID_SET_BG = 0,
+    ID_SET_CAPTION,
     ID_SET_PARAM,
     ID_SET_EFIELD,
     ID_SET_FG,
@@ -239,7 +241,8 @@ enum CommandTimerID {
 
 #define UPLOAD_ICON    "image,icon"
 #define UPLOAD_PIX     "image,window"
-#define UPLOAD_COVER   "cover"
+#define UPLOAD_COVER   "image,cover"
+#define UPLOAD_COVER_DATA "cover,noname"
 
 #define STR_MOTOROLA	"Motorola"
 #define STR_SE   	"Sony Ericsson"
