@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2012-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2012-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,26 +25,29 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "atsend.h"
+#include "btio.h"
 #include "common.h"
 #include "conf.h"
-#include "btio.h"
-#include "atsend.h"
+#include "executor.h"
+#include "gen_ar.h"
+#include "peer.h"
+#include "pr_btspp.h"
+#include "pr_frontend.h"
+#include "pr_l2cap.h"
+#include "pr_rfcomm.h"
+#include "pr_serial.h"
+#include "pr_socket.h"
 #include "pr_stdin.h"
 #include "pr_web.h"
-#include "pr_btspp.h"
-#include "pr_l2cap.h"
-#include "pr_socket.h"
-#include "pr_serial.h"
-#include "pr_rfcomm.h"
-#include "utils.h"
+#include "state.h"
 #include "str.h"
-#include "peer.h"
+#include "utils.h"
 
 extern char tmp[MAXMAXLEN];
 
 static int peerWrite     (ConnectInfo* peer, const dMessage* msg);
 static int peerWriteBytes(ConnectInfo* peer, const char* command);
-static int acceptAT      (ConnectInfo* peer);
 static void writeIViewerHeartbeat(int fd);
 static void writeBemusedHeartbeat(int fd);
 
@@ -55,10 +58,10 @@ static ConnectInfo* allocPeer()
     ConnectInfo* peer = (ConnectInfo*) malloc(sizeof(ConnectInfo));
 
     peer->mode                 = SERVER_MAX;
+    peer->state                = PEER_DISCONNECTED;
     peer->port                 = -1;
     peer->connectionData       = NULL;  // specific to connection
     peer->portStr              = NULL;
-    peer->connected            = BOOL_NO;
 
     return peer;
 }
@@ -73,6 +76,20 @@ static void freePeer(void* data)
     free(peer);
 }
 
+static int countConnections()
+{
+    int cnum = 0;
+    SingleList* list = _connections;
+    while (list) {
+       ConnectInfo* peer = (ConnectInfo*) list->data;
+       if (peer->state == PEER_CONNECTED) {
+           cnum++;
+       }
+       list = listSingleNext(list);
+    }
+    return cnum;
+}
+
 static int dummyOpen(ConnectInfo* connInfo)
 {
     return EXIT_OK;
@@ -83,29 +100,34 @@ static int dummyFD(ConnectInfo* connInfo)
     return -1;
 }
 
-static int acceptNope(ConnectInfo* p)
+static int setupNope(ConnectInfo* p)
 {
-    switch (p->mode) {
-    case SERVER_WEB:
-        logger(L_INF, "[DS]: Built-in web server: init OK");
-        break;
-    case SERVER_CMXML:
-        logger(L_INF, "[DS]: Built-in XML server: init OK");
-        break;
-    case CLIENT_ILIRC:
-    case CLIENT_NOAT:
-    case SERVER_STDIN:
-        logger(L_INF, "[DS]: Unix client: init OK");
-        break;
-    }
     return 1;
 }
 
+static int acceptNope(ConnectInfo* p)
+{
+    switch (p->mode) {
+        case SERVER_WEB:
+            logger(L_INF, "[DS]: Built-in web server: init OK");
+            break;
+        case SERVER_CMXML:
+            logger(L_INF, "[DS]: Built-in XML server: init OK");
+            break;
+        case CLIENT_ILIRC:
+        case CLIENT_NOAT:
+        case SERVER_STDIN:
+            logger(L_INF, "[DS]: Unix client: init OK");
+            break;
+    }
+    return 1;
+}
 
 static struct {
     int  id;
     int  (*descriptor)           (ConnectInfo* p);
     int  (*openConnection)       (ConnectInfo* p);
+    int  (*setupConnection)      (ConnectInfo* p);
     int  (*acceptConnection)     (ConnectInfo* p);
     int  (*writeConnection)      (ConnectInfo* p, const dMessage* msg);
     int  (*writeBytesConnection) (ConnectInfo* p, const char* value);
@@ -113,22 +135,22 @@ static struct {
 
 } _peerHandlers[] = {
 
-    // type         get FD    open            accept        write	  write bytes     close
-    //              (FD/-1)   (EXIT_OK/NOK)   (1/-1)        (EXIT_OK/NOK) (EXIT_OK/NOK)   (void)
-
-    {CLIENT_RFCOMM, rfcommFD, rfcommConnect,  acceptAT,     peerWrite,    NULL, 	      rfcommClose     },
-    {CLIENT_AT,     serialFD, serialOpen,     acceptAT,     peerWrite,    NULL, 	      serialClose     },
-    {SERVER_TCP,    socketFD, socketOpen,     socketAccept, socketWrite,  peerWriteBytes,     socketClose     },
-    {SERVER_BT,     btsppFD,  btsppOpen,      btsppAccept,  peerWrite,    peerWriteBytes,     btsppClose      },
-    {CLIENT_ILIRC,  uxFD,     openUnixSocket, acceptNope,   peerWrite,    NULL, 	      closeUnixSocket },
-    {CLIENT_NOAT,   serialFD, serialOpen,     acceptNope,   peerWrite,    NULL, 	      serialClose     },
-    {SERVER_STDIN,  dummyFD,  openStdin,      acceptNope,   NULL,	  NULL, 	      closeStdin      },
-    {SERVER_WEB,    dummyFD,  openWeb,        acceptNope,   writeWeb,	  NULL, 	      closeWebPort    },
-    {SERVER_CMXML,  dummyFD,  openWeb,        acceptNope,   writeWeb,	  NULL, 	      closeWebPort    },
-#ifdef USE_L2CAP
-    {SERVER_L2CAP,  l2capFD,  l2capOpen,      l2capAccept,  peerWrite,    peerWriteBytes, l2capClose	      },
-#endif
-    {SERVER_UX ,    socketFD, dummyOpen,      socketAccept, peerWrite,    peerWriteBytes, socketClose	      }     // not used
+    // type         get FD    open          setup       accept       write       write bytes    close
+    //              (EXIT_OK  (1/-1)        (1/-1)      (EXIT_OK/   (EXIT_OK/NOK)  (void)
+    //               /NOK)                                   NOK)
+    {CLIENT_RFCOMM, rfcommFD, rfcommConnect,rfcommSetup,acceptNope,  peerWrite,  NULL,          rfcommClose  },
+    {CLIENT_AT,     serialFD, serialOpen,   serialSetup,acceptNope,  peerWrite,  NULL,          serialClose  },
+    {SERVER_TCP,    socketFD, socketOpen,   socketSetup,socketAccept,socketWrite,peerWriteBytes,socketClose  },
+    {SERVER_BT,     btsppFD,  btsppOpen,    btsppSetup, btsppAccept, peerWrite,  peerWriteBytes,btsppClose   },
+    {CLIENT_ILIRC,  uxFD,     openUxSocket, setupNope,  acceptNope,  NULL,       NULL,          closeUxSocket},
+    {CLIENT_NOAT,   serialFD, serialOpen,   setupNope,  acceptNope,  peerWrite,  NULL,          serialClose  },
+    {SERVER_STDIN,  dummyFD,  openStdin,    setupNope,  acceptNope,  NULL,       NULL,          closeStdin   },
+    {SERVER_WEB,    dummyFD,  openWeb,      setupNope,  acceptNope,  writeWeb,   NULL,          closeWebPort },
+    {SERVER_CMXML,  dummyFD,  openWeb,      setupNope,  acceptNope,  writeWeb,   NULL,          closeWebPort },
+    #ifdef USE_L2CAP
+    {SERVER_L2CAP,  l2capFD,  l2capOpen,    l2capSetup, l2capAccept, peerWrite,  peerWriteBytes,l2capClose   },
+    #endif
+    {SERVER_UX ,    socketFD, dummyOpen,    socketSetup,socketAccept,peerWrite,  peerWriteBytes,socketClose  } // not used
 };
 
 void freePeers()
@@ -136,50 +158,52 @@ void freePeers()
     INFO2("[DS]: freePeers() start");
     listSingleFullFree(_connections, freePeer);
     _connections = NULL;
+        
+    freeState();
+
     INFO2("[DS]: freePeers() end");
 }
 
-
 static int addTcpPeer(ConnectInfo* peer, char *strPort)
 {
     // check is it port or unix socket
     char * ch = strPort;
     int isPort = 1;
     while (*ch != '\0') {
-    	if (isdigit(*ch)) {
-    	    ch++;
-    	} else {
-    	    isPort = 0;
-    	    break;
-    	}
+        if (isdigit(*ch)) {
+            ch++;
+        } else {
+            isPort = 0;
+            break;
+        }
     }
 
     if (isPort) {
 
-    	peer->mode = SERVER_TCP;
-    	peer->port = atoi(strPort);
+        peer->mode  = SERVER_TCP;
+        peer->port  = atoi(strPort);
 
-    	if (peer->port <= 0) {
-    	    printf("ERROR: Improper port to use %d!\n", peer->port);
-    	    ERROR2("[DS]: Improper port %d to use", peer->port);
-    	    return EXIT_ABORT;
-    	}
+        if (peer->port <= 0) {
+            printf("ERROR: Improper port to use %d!\n", peer->port);
+            ERROR2("[DS]: Improper port %d to use", peer->port);
+            return EXIT_ABORT;
+        }
 
-    	DEBUG2("[DS]: TCP Server mode, port %d\n", peer->port);
+        DEBUG2("[DS]: TCP Server mode, port %d\n", peer->port);
 
     } else {
 
-    	/*sprintf(tmp, "Unix socket Server mode. Use socket %s\n", strPort);
-    	logger(L_DBG, tmp);
+        /*sprintf(tmp, "Unix socket Server mode. Use socket %s\n", strPort);
+        logger(L_DBG, tmp);
 
-    	if ((ret = openSocketPort(SERVER_UX, -1, strPort)) < 0) {
-    	    return -1;
-    	}
-    	peer->mode = SERVER_UX;
-    	peer->portStr = stringNew(strPort);
-    	*/
-    	printf("ERROR: incorrect port\n");
-    	return EXIT_ABORT;
+        if ((ret = openSocketPort(SERVER_UX, -1, strPort)) < 0) {
+            return -1;
+        }
+        peer->mode = SERVER_UX;
+        peer->portStr = stringNew(strPort);
+        */
+        printf("ERROR: incorrect port\n");
+        return EXIT_ABORT;
 
     }
     return EXIT_OK;
@@ -188,7 +212,7 @@ static int addTcpPeer(ConnectInfo* peer, char *strPort)
 //
 // fill ConnectInfo structure, return EXIT_ABORT in unsuccessful
 //
-int addPeer(char * portIn)
+static int addPeer(char * portIn)
 {
 
     if (portIn == NULL) {
@@ -201,17 +225,17 @@ int addPeer(char * portIn)
     // device should be in format:
 
     // deprecated
-    // 1. (AT mode)     /dev/something							used with AT mode commands
+    // 1. (AT mode)     /dev/something                            used with AT mode commands
 
     // 1. (AT mode)     rfcomm:00:12:EF:32:21:1A:xx (xx is integer from 1 to 32)
-    // 2. (Server mode) socket:NNNN or 							used with java client
-    //                  socket:/path/to/socket						used with java client (SERVER_UX, not used)
-    // 3. (Server mode) bluetooth:NN or bluetooth (port will be = DEFAULT_BT_CHANNEL) 	used with java client
-    // 4. (Server mode) web:NNNN							built-in web server
-    // 5. (Server mode) cmxml:NNNN							XML services interface
-    // 6. local:/some/path								used with java client, like 1, but no AT commands
-    // 7. ilirc:/dev/something 								like 1, but no AT commands
-    // 8. stdio								                used with java client, like 1, but no AT commands
+    // 2. (Server mode) socket:NNNN or                             used with java client
+    //                  socket:/path/to/socket                    used with java client (SERVER_UX, not used)
+    // 3. (Server mode) bluetooth:NN or bluetooth (port will be = DEFAULT_BT_CHANNEL)     used with java client
+    // 4. (Server mode) web:NNNN                            built-in web server
+    // 5. (Server mode) cmxml:NNNN                            XML services interface
+    // 6. local:/some/path                                    used with java client, like 1, but no AT commands
+    // 7. ilirc:/dev/something                                 like 1, but no AT commands
+    // 8. stdio                                                used with java client, like 1, but no AT commands
 
 
     ConnectInfo* peer = allocPeer();
@@ -219,26 +243,26 @@ int addPeer(char * portIn)
     if ((strPort = strstr(portIn, INET_SOCKET)) != NULL) {
 
         strPort += strlen(INET_SOCKET);
-	
-	if (addTcpPeer(peer, strPort) == EXIT_ABORT) {
+    
+        if (addTcpPeer(peer, strPort) == EXIT_ABORT) {
             free(peer);
             return EXIT_ABORT;
-	}
-	
+        }
+    
     } else if ((strPort = strstr(portIn, PEER_TCP)) != NULL) {
 
         strPort += strlen(PEER_TCP);
-	
-	if (addTcpPeer(peer, strPort) == EXIT_ABORT) {
+    
+        if (addTcpPeer(peer, strPort) == EXIT_ABORT) {
             free(peer);
             return EXIT_ABORT;
-	}
-	
+        }
+    
     } else if ((strPort = strstr(portIn, ILIRC_SOCKET)) != NULL) {
 
         strPort += strlen(ILIRC_SOCKET);
 
-        peer->mode = CLIENT_ILIRC;
+        peer->mode  = CLIENT_ILIRC;
         peer->portStr = stringNew(strPort);
 
         DEBUG2("[DS]: Unix socket client mode, socket %s\n", peer->portStr->str);
@@ -253,32 +277,29 @@ int addPeer(char * portIn)
         } else {
             strPort++;
             peer->port = atoi(strPort);
-
-
         }
         DEBUG2("[DS]: Bluetooth Sockets Server mode, channel %d", peer->port);
 
     } else if ((strPort = strstr(portIn, L2CAP_SOCKET)) != NULL) {
 
-#ifdef USE_L2CAP
+        #ifdef USE_L2CAP
         strPort += strlen(L2CAP_SOCKET);
 
         peer->mode = SERVER_L2CAP;
         if (strstr(strPort, ":") == NULL) { // just "l2cap"
             peer->port = DEFAULT_L2CAP_PORT;
         } else {
-            strPort++;
             peer->port = atoi(strPort);
         }
         DEBUG2("[DS]: L2CAP Server mode, port %d", peer->port);
 
-#endif
+        #endif
 
     } else if ((strPort = strstr(portIn, UNIX_SOCKET)) != NULL) {
 
         strPort += strlen(UNIX_SOCKET);
 
-        peer->mode = CLIENT_NOAT;
+        peer->mode    = CLIENT_NOAT;
         peer->portStr = stringNew(strPort);
 
         DEBUG2("[DS]: Serial Client mode (no AT). Use device %s", peer->portStr->str);
@@ -302,8 +323,8 @@ int addPeer(char * portIn)
         strncpy(sBtAddr,portIn + strlen(RFCOMM_DEVICE) + 1,17);
         sBtAddr[17] = '\0';
 
-        peer->mode = CLIENT_RFCOMM;
-        peer->port = atoi(portIn + strlen(RFCOMM_DEVICE) + BT_ADDR_LEN + 2);
+        peer->mode  = CLIENT_RFCOMM;
+        peer->port  = atoi(portIn + strlen(RFCOMM_DEVICE) + BT_ADDR_LEN + 2);
         peer->portStr = stringNew(sBtAddr);
 
         DEBUG2("[DS]: Serial Client mode, device %s : %d", peer->portStr->str, peer->port);
@@ -326,8 +347,8 @@ int addPeer(char * portIn)
 
         if (isPort) {
 
-            peer->mode = SERVER_WEB;
-            peer->port = atoi(strPort);
+            peer->mode  = SERVER_WEB;
+            peer->port  = atoi(strPort);
             if (peer->port <= 0) {
                 printf("ERROR: Improper port to use !\n");
                 ERROR2("[DS]: Improper port %d to use", peer->port);
@@ -361,8 +382,8 @@ int addPeer(char * portIn)
 
         if (isPort) {
 
-            peer->mode = SERVER_CMXML;
-            peer->port = atoi(strPort);
+            peer->mode  = SERVER_CMXML;
+            peer->port  = atoi(strPort);
 
             DEBUG2("[DS]: XML Server mode. Use port %d", peer->port);
 
@@ -376,13 +397,13 @@ int addPeer(char * portIn)
 
         strPort += strlen(AT_DEVICE);
 
-        peer->mode = CLIENT_AT;
+        peer->mode  = CLIENT_AT;
         peer->portStr = stringNew(strPort);
         DEBUG2("[DS]: Serial Client mode. Use device %s", peer->portStr->str);
 
     } else {
 
-        peer->mode = CLIENT_AT;
+        peer->mode  = CLIENT_AT;
         peer->portStr = stringNew(portIn);
         DEBUG2("[DS]: Default: use serial Client mode. Use device %s", peer->portStr->str);
     }
@@ -392,6 +413,38 @@ int addPeer(char * portIn)
     return EXIT_OK;
 }
 
+void connectNotify()
+{
+    sendToFrontEnd("Connected");
+    sendEventToExecutor(ID_EVT_CONNECT);
+}
+
+static boolean_t needStoreState()
+{
+    int connCnt = 0;
+    
+    SingleList* list = _connections;
+    while (list) {
+        ConnectInfo* peer = (ConnectInfo*) list->data;
+
+        if (peer->mode == SERVER_WEB || 
+            peer->mode == SERVER_CMXML) {
+            connCnt += 2;   // need to store state in any case if WEB/CMXML is used
+        } else if (peer->mode == SERVER_TCP || 
+                   peer->mode == SERVER_BT  ||
+                   peer->mode == CLIENT_NOAT||
+                   peer->mode == SERVER_UX  ||
+                  peer->mode ==  SERVER_L2CAP) {
+            connCnt += 1;
+        }
+        if (connCnt > 1) {
+            break;
+        }
+        list = listSingleNext(list);
+    }
+    return (connCnt > 1 ? BOOL_YES : BOOL_NO);
+}
+
 int definePeers()
 {
     DEBUG2("[DS]: definePeers");
@@ -422,6 +475,10 @@ int definePeers()
     }
 
     free(peers);
+    
+    if (needStoreState()) {
+        initState();
+    }
 
     DEBUG2("[DS]: definePeers done %d (peers #%d)", ret, listSingleLength(_connections));
     return ret;
@@ -435,15 +492,19 @@ int openPeers()
     SingleList* list = _connections;
     while (list) {
         ConnectInfo* peer = (ConnectInfo*) list->data;
+        
+        if (peer->state == PEER_DISCONNECTED) {
+            INFO2("[DS]: Open peer connect mode %d", peer->mode);
 
-        INFO2("[DS]: Open peer connect mode %d", peer->mode);
+            int ret1 = _peerHandlers[peer->mode].openConnection(peer);
 
-        int ret1 =_peerHandlers[peer->mode].openConnection(peer);
+            INFO2("[DS]: Open peer connect mode %d fdescriptor %d", peer->mode, _peerHandlers[peer->mode].descriptor(peer));
 
-        INFO2("[DS]: Open peer connect mode %d fdescriptor %d", peer->mode, _peerHandlers[peer->mode].descriptor(peer));
-
-        if (ret1 != EXIT_NOK) {
-            ret = ret1;
+            if (ret1 != EXIT_NOK) {
+                ret = ret1;
+            }
+        } else {
+            DEBUG2("[DS]: openPeers peer with mode %d have improper state %d", peer->mode, peer->state);
         }
         list = listSingleNext(list);
     }
@@ -475,19 +536,19 @@ int disconnectPeersInternal(ConnectInfo* peer)
     if (peer->mode == SERVER_BT) {
 
         if (getIViewer() || getBemused()) { 
-	    canForceExit = 1;
-	} else {
+            canForceExit = 1;
+        } else {
             logger(L_INF, "[DS]: Got exit event: send disconnect message");
             if (btsppWrite(peer, dm) != EXIT_OK) {
-        	canForceExit = 1;
+                canForceExit = 1;
             }
         }
 
     } else if (peer->mode == SERVER_TCP) {
 
         if (getIViewer() || getBemused()) { 
-	    canForceExit = 1;
-	} else {
+            canForceExit = 1;
+        } else {
             logger(L_INF, "[DS]: Got exit event: send disconnect message");
             if (socketWrite(peer, dm) != EXIT_OK) {
                 canForceExit = 1;
@@ -550,33 +611,20 @@ int disconnectPeers(void)
     return ret;
 }
 
-static int acceptAT(ConnectInfo* peer)
-{
-    int fd = _peerHandlers[peer->mode].descriptor(peer);
-    if (fd < 0) {
-        DEBUG2("acceptAT() wrong descriptor");
-        return -1;
-    }
-
-    return setupAtConnection(peer, fd);
-}
-
 // Support single socket connection + web server at once for now
-int acceptPeers(void)
+int setupPeers(void)
 {
-    DEBUG2("[DS]: acceptPeers");
+    DEBUG2("[DS]: setupPeers");
     int ret = -1;
     SingleList* list = _connections;
     while (list) {
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
+        
+        int retS = _peerHandlers[peer->mode].setupConnection(peer);
 
-        INFO2("[DS]: Accept connection connect mode %d", peer->mode);
-
-        int ret1 = _peerHandlers[peer->mode].acceptConnection(peer);
-
-        if (ret1 != -1) {
-            ret = ret1;
+        if (retS != -1) {
+            ret = retS;
         }
         list = listSingleNext(list);
     }
@@ -618,6 +666,78 @@ static int peerWrite(ConnectInfo* peer, const dMessage* msg)
 
 }
 
+//
+// Sync peer after connect
+//
+static void syncPeer(ConnectInfo* peer)
+{
+    if (peer->mode == SERVER_TCP  ||
+        peer->mode == SERVER_BT   ||
+        peer->mode == SERVER_UX   ||
+        peer->mode == CLIENT_NOAT ||
+        peer->mode == SERVER_L2CAP) {
+        
+        ERROR2("[DS]: syncPeer %d", peer->mode);
+
+        static struct {
+           string_t* (*hooks[6]) (int) ; // CF,TX,LI,FM,WM,EF
+        } _renderHooks[] = {
+           //                  CF                  TX                  LI                  FM    WM                  EF 
+           /* SERVER_TCP+  */{{renderCtrlForm,     renderTextForm,     renderListForm,     NULL, renderWmanForm,     renderEditForm}}
+        };
+
+         
+        int f   = curForm() - 1;
+        INFO2("[DS]: syncPeer form %d", f);
+    
+        if (_renderHooks[0].hooks[f]) {
+            string_t* content = _renderHooks[0].hooks[f](peer->port);
+	        if (content) {
+                
+                //char buf[256];
+                //strncpy(buf,content->str,255);
+                //buf[255] = '\0';
+                //INFO2("[DS]: syncPeer sync content %s", buf);
+                
+                int fd = _peerHandlers[peer->mode].descriptor(peer);
+                if (fd >= 0) {
+                    writePeer(fd, content->str, content->len);
+                }
+	            stringFree(content, BOOL_YES);
+	        }
+        }
+        
+        if (curForm() == CF) {  // take care about cover 
+
+            int fd = _peerHandlers[peer->mode].descriptor(peer);
+            if (fd >= 0) {
+                
+                string_t* page = renderCtrlFormCover();
+                writePeer(fd, page->str, page->len);
+	            stringFree(page, BOOL_YES);
+
+                const char* nc = cfNamedCover();
+                const char* cv = cfCover();
+                
+                if (!nc && cv) {
+                
+                    string_t* page = stringNew("Set(cover,noname,");
+                    stringAppend(page, cv);
+                    stringAppend(page, ");");
+               
+                    eMessage* em = (eMessage*) malloc(sizeof(eMessage));
+                    em->type  = EM_STRING;
+                    em->value = strdup(page->str);
+                    
+                    stringFree(page, BOOL_YES);
+                    
+                    sendToExecutor(em);
+                }
+            }
+        }
+    }
+}
+
 //
 // Default peer reader
 //
@@ -625,7 +745,7 @@ int readPeer(int fd, char* buffer, int max)
 {
     int nbytes = read(fd, buffer, max);
     if (nbytes < 0)  { // Read error
-        ERROR2("[DS]: readPeer() error %d",errno);
+        //ERROR2("[DS]: readPeer() error %d",errno);
         errnoDebug("[DS]: readPeer() read ",errno);  // testing debug
     } else if (nbytes == 0) {
         DEBUG2("[DS]: readPeer() EOF");
@@ -640,7 +760,7 @@ int readPeer(int fd, char* buffer, int max)
 //
 static int readIVPeer(int fd, char* buffer, int max)
 {
- 	//DEBUG2("[DS]: readIVPeer()");
+     //DEBUG2("[DS]: readIVPeer()");
     int nbytes = read(fd, buffer, max);
 
     
@@ -655,7 +775,7 @@ static int readIVPeer(int fd, char* buffer, int max)
         //DEBUG2("[DS]: readIVPeer() got %s", buffer);
         
         //while ((hptr = strstr(hptr, "h=0"))) {
-		if (strstr(hptr, "h=0")) {
+        if (strstr(hptr, "h=0")) {
             // do not erase it from read data, because it used as disconnect timer in cfg.files
             /*
             *hptr = '\r';
@@ -795,40 +915,80 @@ static int readPeersInternal(ConnectInfo* peer, string_t* buffer)
         }
 
     } else {
-
+        
         /* use select() for others
-        char* p = buf;
+        int fd = _peerHandlers[peer->mode].descriptor(peer);
+        if (peer->state == PEER_CONNECTED && fd >= 0) {
+        
+            char* p = buf;
 
-        	while (len < MAXCMDLEN) {
-        	    char c;
-        	    ch = bt_readchar(&c,100);
+            while (len < MAXCMDLEN) {
+                char c;
+                int ch = bt_readchar(fd,&c,100);
+                
+                INFO2("[DS]: readPeersInternal read from peer with mode %d got %d", peer->mode, ch);
+                
+                if (ch == EOF) {
+                    INFO2("[DS]: readPeersInternal EOF peer with mode %d", peer->mode);
+                    buf[0] = 0;
+                    peer->state = PEER_DISCONNECTED;
+                    break;
+                } else if (ch == EOF-1) {
+                    break;
+                } else if (ch >= 0 && (c == '\r' || c == ';')) {
+                    break;
+                } else  if (ch >= 0 && c != '\r' && c != ';') {
+                    *p++ = c;
+                    len++;
+                }
+            }
 
-        	    if (ch == EOF) {
-        		buf[0] = 0;
-        		peer->connected = BOOL_NO;
-        	break;
-        	    } else if (ch == EOF-1) {
-        		break;
-        	    } else if (ch >= 0 && (c == '\r' || c == ';')) {
-        		break;
-        	    } else  if (ch >= 0 && c != '\r' && c != ';') {
-        		*p++ = c;
-        		len++;
-        	    }
-        	}
-
-        buf[len] = '\0';
-        stringAppend(buffer,"\r");  // separator
-        stringAppend(buffer,buf);
+            buf[len] = '\0';
+            stringAppend(buffer,"\r");  // separator
+            stringAppend(buffer,buf);
+        }
         */
+        
     }
     return len;
 }
 
+static int doReadPeer(ConnectInfo* peer, string_t* buffer, int fd)
+{
+    char buf[MAXCMDLEN];
+    
+    //DEBUG2("[DS]: doReadPeer() ready to read from fd %d (peer mode %d)", fd, peer->mode);
+
+    int rc = 0;
+    
+    if (peer->mode == SERVER_TCP && getIViewer()) {
+        rc = readIVPeer(fd, buf, MAXCMDLEN);
+    } else if (peer->mode == SERVER_BT && getBemused()) {
+        rc = readBmPeer(fd, buf, MAXCMDLEN);
+    } else if (peer->mode == CLIENT_NOAT) {
+        rc = readIRPeer(fd, buf, MAXCMDLEN);
+    } else if (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT) {
+        rc = atRead(peer, buf, MAXCMDLEN);
+    } else {
+        rc = readPeer(fd, buf, MAXCMDLEN);
+    }
+
+    if (rc <= 0) {  // EOF or error
+        DEBUG2("[DS]: doReadPeer() EOF or error fd %d (peer mode %d), close connection", fd, peer->mode);
+        _peerHandlers[peer->mode].closeConnection(peer, 0);
+        return EOF;
+    } else {
+        stringAppend(buffer,"\r");  // separator
+        stringAppendLen(buffer,buf,rc);
+    }
+
+    return 1;
+}
+
 int readPeers(string_t* buffer)
 {
     //DEBUG2("[DS]: readPeers()");
-
+    
     fd_set read_fds;
     FD_ZERO(&read_fds);
 
@@ -841,17 +1001,23 @@ int readPeers(string_t* buffer)
     SingleList* list = _connections;
     while (list) {
         ConnectInfo* peer = (ConnectInfo*) list->data;
-        if (peer->connected == BOOL_YES) {
-            //DEBUG2("[DS]: readPeers() connected %d", peer->mode);
+
+        if ((peer->state == PEER_CONNECTED && 
+             !(peer->mode == SERVER_STDIN || peer->mode == SERVER_WEB || peer->mode == SERVER_CMXML)) ||
+            peer->state == PEER_WAIT_ACCEPT) {
 
             int fd = _peerHandlers[peer->mode].descriptor(peer);
+            //DEBUG2("[DS]: readPeers() peer with mode %d connected %s state (fd=%d)", peer->mode, 
+            //             (peer->state == PEER_CONNECTED ? "CONNECTED" : "WAIT_ACCEPT"),fd);
+                         
             if (fd >= 0) {
                 FD_SET(fd, &read_fds);
+            //} else {  web, xml can have that 
+            //    DEBUG2("[DS]: readPeers() peer with mode %d improper descriptor", peer->mode); 
             }
             if (max_fd < fd) {
                 max_fd = fd;
             }
-
         }
         list = listSingleNext(list);
     }
@@ -859,52 +1025,70 @@ int readPeers(string_t* buffer)
     if (max_fd >= 0) {  // have some peers
 
         int rc = select(max_fd + 1, &read_fds, NULL, NULL, &tv);
-
+        
+        //if (rc != 0) {
+        //    DEBUG2("[DS]: readPeers() select()=%d, maxfd=%d",rc,max_fd);
+        //}
+        
         if (rc == 0) {          // timeout
             //DEBUG2("[DS]: readPeers() no data from select()");
         } else if (rc < 0) {    // some error
-            DEBUG2("[DS]: readPeers() error %d", errno);
+            //DEBUG2("[DS]: readPeers() error %d", errno);
             errnoDebug("[DS]: readPeers() select ",errno);  // testing debug
         } else {
-
-            char buf[MAXCMDLEN];
-
+            //DEBUG2("[DS]: readPeers() got data from select()");
+            
             list = _connections;
             while (list) {
+             
                 ConnectInfo* peer = (ConnectInfo*) list->data;
-                if (peer->connected == BOOL_YES) {
-                    int fd = _peerHandlers[peer->mode].descriptor(peer);
-                    if (fd >= 0) {
-                        if (FD_ISSET(fd, &read_fds)) {
-                            DEBUG2("[DS]: readPeers() ready to read from fd %d (peer mode %d)", fd, peer->mode);
+                
+                int fd = _peerHandlers[peer->mode].descriptor(peer);
+                if (fd >= 0) {
+                
+                    //DEBUG2("[DS]: readPeers() %d descriptor on peer with mode %d", fd, peer->mode);
 
+                    if (FD_ISSET(fd, &read_fds)) {
 
-                            if (peer->mode == SERVER_TCP && getIViewer()) {
-                                rc = readIVPeer(fd, buf, MAXCMDLEN);
-                            } else if (peer->mode == SERVER_BT && getBemused()) {
-                                rc = readBmPeer(fd, buf, MAXCMDLEN);
-                            } else if (peer->mode == CLIENT_NOAT) {
-                                rc = readIRPeer(fd, buf, MAXCMDLEN);
-                            } else if (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT) {
-			        rc = atRead(peer, buf, MAXCMDLEN);
-                            } else {
-                                rc = readPeer(fd, buf, MAXCMDLEN);
+                        if (peer->state == PEER_CONNECTED) {
+                            //DEBUG2("[DS]: readPeers() peer with mode %d readable", peer->mode);
+                            int rr = doReadPeer(peer, buffer, fd);
+                            if (rr == EOF) {
+                                int cnum = countConnections();
+
+                                // no more connections
+                                if (cnum == 0) { 
+                                     DEBUG2("[DS]: readPeers() disconnect notify");
+                                     return EOF;
+                                }
                             }
+                        } else if (peer->state == PEER_WAIT_ACCEPT) {
 
-                            if (rc <= 0) {  // EOF or error
-                                DEBUG2("[DS]: readPeers() EOF or error fd %d (peer mode %d), close connection", fd, peer->mode);
-                                _peerHandlers[peer->mode].closeConnection(peer, 0);
-                                return EOF;
-                            } else {
-                                stringAppend(buffer,"\r");  // separator
-                                stringAppendLen(buffer,buf,rc);
+                            //DEBUG2("[DS]: readPeers() accept connection peer with mode %d", peer->mode);
+                            if (_peerHandlers[peer->mode].acceptConnection(peer) == EXIT_OK) {
+                                int cnum = countConnections();
+
+                                // do this only on first connect
+                                if (cnum == 1) {
+                                    DEBUG2("[DS]: readPeers() connectNotify");
+                                    connectNotify();
+                                } else {
+                                    freeCachedData();
+                                    syncPeer(peer);
+                                }
                             }
                         }
+                    //} else {
+                    //    DEBUG2("[DS]: readPeers() no FD_ISSET on peer with mode %d", peer->mode);
                     }
+                //} else {
+                //   DEBUG2("[DS]: readPeers() no descriptor on peer with mode %d", peer->mode);
                 }
                 list = listSingleNext(list);
             }
         }
+    //} else {
+    //    DEBUG2("[DS]: readPeers() no peers to read");
     }
 
     //DEBUG2("[DS]: readPeers() special cases");
@@ -912,13 +1096,16 @@ int readPeers(string_t* buffer)
     list = _connections;
     while (list) {
         ConnectInfo* peer = (ConnectInfo*) list->data;
-        if (peer->connected == BOOL_YES) {
+        if (peer->state == PEER_CONNECTED) {
             int rc = readPeersInternal(peer, buffer);
             if (rc < 0 || rc == EOF) {  // EOF or error
                 DEBUG2("[DS]: readPeers() EOF or error special case (peer mode %d), close connection", peer->mode);
                 _peerHandlers[peer->mode].closeConnection(peer, 0);
                 return EOF;
             }
+        // Only stdin, web and cmxml goes here. None of them can have state PEER_WAIT_ACCEPT
+        //} else if (peer->state == PEER_WAIT_ACCEPT) {
+        //    //DEBUG2("[DS]: readPeers() TODO aa1");
         }
         list = listSingleNext(list);
     }
@@ -930,11 +1117,16 @@ int readPeers(string_t* buffer)
 int writePeers(dMessage* dm)
 {
     int ret = EXIT_NOK;
+    
+    if (needStoreState()) {
+        updateState(dm);
+    }
+
     SingleList* list = _connections;
     while (list) {
         ConnectInfo* peer = (ConnectInfo*) list->data;
 
-        if (peer->connected == BOOL_YES) {
+        if (peer->state == PEER_CONNECTED) {
             INFO2("[DS]: write to peer %d", peer->mode);
 
             int ret1 = (_peerHandlers[peer->mode].writeConnection != NULL ?
@@ -980,12 +1172,12 @@ static int peerWriteBytes(ConnectInfo* peer, const char* command)
 
         strncpy(byteStr,command,MAXCKPDLEN-1);
 
-	DEBUG2("[DS]: peerWriteBytes >%s<", byteStr); 
+        DEBUG2("[DS]: peerWriteBytes >%s<", byteStr); 
 
         char* bStr = strtok(byteStr,",");
         while (bStr != NULL) {
 
-	    //DEBUG2("[DS]: Next byte is >%s<", bStr); 
+        //DEBUG2("[DS]: Next byte is >%s<", bStr); 
 
             char bStripped[4];
 
@@ -1000,7 +1192,7 @@ static int peerWriteBytes(ConnectInfo* peer, const char* command)
             }
             bStripped[i] = '\0';
 
-	    //DEBUG2("[DS]: Next byte is >%s<", bStripped); 
+            //DEBUG2("[DS]: Next byte is >%s<", bStripped); 
 
             if (writeByteInternal(fd, atoi(bStripped)) != EXIT_OK) {
                 logger(L_DBG,"[DS]: Fails in peerWriteBytes()");
@@ -1020,7 +1212,9 @@ int writeBytesPeers(char* command)
     SingleList* list = _connections;
     while (list) {
         ConnectInfo* peer = (ConnectInfo*) list->data;
-        if (peer && peer->connected == BOOL_YES && _peerHandlers[peer->mode].writeBytesConnection != NULL) {
+        if (peer && 
+            peer->state == PEER_CONNECTED &&
+            _peerHandlers[peer->mode].writeBytesConnection != NULL) {
 
             INFO2("[DS]: write bytes to peer %d", peer->mode);
             int ret1 = _peerHandlers[peer->mode].writeBytesConnection(peer, command);
@@ -1040,6 +1234,10 @@ int writeBytesPeers(char* command)
 int writeFilePeers(dMessage* dm)
 {
     INFO2("[DS]: DM_SETFILE");
+    
+    if (needStoreState()) {
+        updateState(dm);
+    }
 
     int  size = 0;
     char* buf = NULL;
@@ -1051,12 +1249,12 @@ int writeFilePeers(dMessage* dm)
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
 
-        if (peer->connected == BOOL_YES) {
+        if (peer->state == PEER_CONNECTED) {
             INFO2("[DS]: write file to peer %d", peer->mode);
 
             if (peer->mode == SERVER_WEB || peer->mode == SERVER_CMXML) {
 
-                writeWebConn(dm, peer->mode);
+                // nothing, state stored by updateState(...) above
 
             } else {
 
@@ -1103,8 +1301,8 @@ int writeCKPD(dMessage* dm)
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
 
-        if (peer->connected == BOOL_YES &&
-                (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT)) {
+        if (peer->state == PEER_CONNECTED &&
+            (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT)) {
 
             int fd = _peerHandlers[peer->mode].descriptor(peer);
             if (fd >= 0) {
@@ -1125,8 +1323,8 @@ int writeCMER(dMessage* dm)
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
 
-        if (peer->connected == BOOL_YES &&
-                (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT)) {
+        if (peer->state == PEER_CONNECTED &&
+            (peer->mode == CLIENT_RFCOMM || peer->mode == CLIENT_AT)) {
 
             int fd = _peerHandlers[peer->mode].descriptor(peer);
             if (fd >= 0) {
@@ -1147,7 +1345,7 @@ int connected()
     while (list) {
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
-        if (peer->connected == BOOL_YES) {
+        if (peer->state == PEER_CONNECTED) {
             return EXIT_OK;
         }
         list = listSingleNext(list);
@@ -1201,7 +1399,9 @@ void sendIViewerHeartbeat(void)
     while (list) {
 
         ConnectInfo* peer = (ConnectInfo*) list->data;
-        if (peer->connected == BOOL_YES && peer->mode == SERVER_TCP) {  // iViewer connection can be TCP only
+        if (peer->state == PEER_CONNECTED && 
+            peer->mode == SERVER_TCP) {  // iViewer connection can be TCP only
+            
             writeHeartbeat(peer);
         }
         list = listSingleNext(list);
@@ -1217,13 +1417,13 @@ int needExit()
     while (list) {
         ConnectInfo* cn = (ConnectInfo*) list->data;
 
-        if (cn->connected == BOOL_YES) {
+        if (cn->state == PEER_CONNECTED) {
             return EXIT_NOK;
         }
 
         if (!(cn->mode == CLIENT_RFCOMM ||
-                cn->mode == CLIENT_AT     ||
-                cn->mode == CLIENT_ILIRC)) {
+              cn->mode == CLIENT_AT     ||
+              cn->mode == CLIENT_ILIRC)) {
             return EXIT_NOK;
         }
         list = listSingleNext(list);
@@ -1378,13 +1578,13 @@ boolean_t checkActiveCall()
         ConnectInfo* cn = (ConnectInfo*) list->data;
 
         if (cn->mode == CLIENT_RFCOMM) {
-	    if (rfcommCheckActiveCall(cn)) {
-	        return BOOL_YES;
-	    }
+        if (rfcommCheckActiveCall(cn)) {
+            return BOOL_YES;
+        }
         } else if (cn->mode == CLIENT_AT) { 
-	    if (serialCheckActiveCall(cn)) {
-	        return BOOL_YES;
-	    }
+        if (serialCheckActiveCall(cn)) {
+            return BOOL_YES;
+        }
         }
         list = listSingleNext(list);
     }
@@ -1398,13 +1598,13 @@ boolean_t hasActiveCall()
         ConnectInfo* cn = (ConnectInfo*) list->data;
 
         if (cn->mode == CLIENT_RFCOMM) {
-	    if (rfcommHasActiveCall(cn)) {
-	        return BOOL_YES;
-	    }
+        if (rfcommHasActiveCall(cn)) {
+            return BOOL_YES;
+        }
         } else if (cn->mode == CLIENT_AT) { 
-	    if (serialHasActiveCall(cn)) {
-	        return BOOL_YES;
-	    }
+        if (serialHasActiveCall(cn)) {
+            return BOOL_YES;
+        }
         }
         list = listSingleNext(list);
     }
