@@ -22,11 +22,13 @@
 #ifndef _PR_FRONTEND_H
 #define _PR_FRONTEND_H
 
+#include "lib_wrapper.h"
 
 // Frontend support
-gboolean frontendRoutine(gpointer data);
+boolean_t frontendRoutine(pointer_t data);
 int   connectFrontEnd	(int portno);
 void  sendToFrontEnd	(const char *buf);
+void  freeFMessage	(void *buf);
 void  disconnectFrontEnd(void);
 int   readFromFrontEnd	(const char *buf, int size);
 
