@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2009-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2009-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,8 +55,8 @@ typedef struct _L2CapConnection_ {
     sdp_record_t  *record;
     #endif
     #ifdef USE_BT_FBSD
-    void	  *session = NULL;
-    uint32_t	  record;
+    void      *session = NULL;
+    uint32_t      record;
     #endif
 } _L2CapConnection;
 
@@ -266,12 +266,25 @@ void l2capClose(ConnectInfo* connInfo, int final)
     
     if (final) {
         sdpDeregisterL2cap(cn);
-	free(cn);
-	connInfo->connectionData = NULL;
-	connInfo->connected = BOOL_NO;
     }
+    
+    free(cn);
+    connInfo->connectionData = NULL;
+    connInfo->state = PEER_DISCONNECTED;
 }
 
+int l2capSetup(ConnectInfo* connInfo)
+{
+    _L2CapConnection* cn = (_L2CapConnection*) connInfo->connectionData;
+    if (!cn) {
+        return -1;
+    }
+    int ret = listen(cn->serverFileDescriptor,0);
+    if (ret >= 0) {
+        conn->state = PEER_WAIT_ACCEPT;
+    }
+    return (ret < 0 ? -1 : 1);
+}
 
 int l2capAccept(ConnectInfo* connInfo)
 {
@@ -293,7 +306,6 @@ int l2capAccept(ConnectInfo* connInfo)
     if (!cn) return -1;
 
     while (stillRun) {
-        listen(cn->serverFileDescriptor,0);
 
         cn->fileDescriptor = accept(cn->serverFileDescriptor, (struct sockaddr *)&l2rem_addr, &opt);
 
@@ -318,41 +330,51 @@ int l2capAccept(ConnectInfo* connInfo)
         }
 
         ba2str(&l2rem_addr.l2_bdaddr, buf);
-	
-	if (getUsePassword()) {
-	    logger(L_DBG,"[DS]: l2capAccept: Do password verification");
-	    
-	    int ret = EXIT_OK;
-            
-	    int i=0;
-	    for ( ; i<3; i++) {
-	    
-		ret = verifyPassword(cn->fileDescriptor);
+    
+        if (!isAllowed(buf)) {
+            INFO2("l2capAccept: host %s is not in the list of accepted host, close connection", buf);
+            write(cn->fileDescriptor,CMD_STR_DISCONNECT,strlen(CMD_STR_DISCONNECT));
+        
+            close(cn->fileDescriptor);
+            cn->fileDescriptor = -1;
+            connInfo->state = PEER_DISCONNECTED;
+        
+            return -1;
+        }
 
-		if (ret == EXIT_OK) {    // got it
-		   break;
-		}
-	    }
-	    
-	    if (ret != EXIT_OK) {
+        if (getUsePassword()) {
+            logger(L_DBG,"[DS]: l2capAccept: Do password verification");
 
-	        if (ret == EXIT_NOK) {
-		    write(cn->fileDescriptor,CMD_STR_DISCONNECT,strlen(CMD_STR_DISCONNECT));
-		}
-		
-		close(cn->fileDescriptor);
-        	cn->fileDescriptor = -1;
-		connInfo->connected = BOOL_NO;
-		
-		return -1;
-	    }
-	    logger(L_DBG,"[DS]: l2capAccept: Password verification OK");
-	}
-	
+            int ret = EXIT_OK;
+
+            int i=0;
+            for ( ; i<3; i++) {
+                ret = verifyPassword(cn->fileDescriptor);
+
+                if (ret == EXIT_OK) {    // got it
+                   break;
+                }
+            }
+
+            if (ret != EXIT_OK) {
+
+                if (ret == EXIT_NOK) {
+                    write(cn->fileDescriptor,CMD_STR_DISCONNECT,strlen(CMD_STR_DISCONNECT));
+                }
+
+                close(cn->fileDescriptor);
+                cn->fileDescriptor = -1;
+                connInfo->state = PEER_DISCONNECTED;
+
+                return -1;
+            }
+            logger(L_DBG,"[DS]: l2capAccept: Password verification OK");
+        }
+    
         sprintf(tmp, "l2capAccept: accepted from %s", buf);
         logger(L_INF, tmp);
-	
-	connInfo->connected = BOOL_YES;
+    
+        connInfo->state = PEER_CONNECTED;
 
         break;
     }
@@ -363,8 +385,8 @@ int l2capWrite(ConnectInfo* connInfo, dMessage* msg)
 {
     _L2CapConnection* cn = (_L2CapConnection*) connInfo->connectionData;
     if (!cn) {    
-	logger(L_DBG,"[DS]: l2capWrite() no connection data");
-	return EXIT_NOK;
+        logger(L_DBG,"[DS]: l2capWrite() no connection data");
+        return EXIT_NOK;
     }
     const char* command = msg->value; 
     int count = msg->size; 
@@ -395,8 +417,8 @@ int l2capWrite(ConnectInfo* connInfo, dMessage* msg)
 
     int n = write(cn->fileDescriptor,command,count);
     if (n < 0) {
-    	logger(L_ERR, "error writing to L2CAP socket");
-    	return EXIT_NOK;
+        logger(L_ERR, "error writing to L2CAP socket");
+        return EXIT_NOK;
     }
 
     return EXIT_OK;
