@@ -33,7 +33,11 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <unistd.h>
+#ifdef __FreeBSD__
+#include <sys/uio.h>
+#else
 #include <sys/sendfile.h>
+#endif
 
 #define SERVER     "webserver/1.1"
 #define PROTOCOL   "HTTP/1.1"
@@ -116,42 +120,40 @@ static void sendToWebServer(wMessage *buf)
     }
 }
 
+static struct {
+    char* ext;
+    char* mime;
+} _mimeTypes[] = {
+    {".html", "text/html"},
+    {".htm",  "text/html"},
+    {".jpg",  "image/jpeg"},
+    {".jpeg", "image/jpeg"},
+    {".gif",  "image/gif"},
+    {".png",  "image/png"},
+    {".css",  "text/css"},
+    {".au",   "audio/basic"},
+    {".wav",  "audio/wav"},
+    {".avi",  "video/x-msvideo"},
+    {".mpeg", "video/mpeg"},
+    {".mpg",  "video/mpeg"},
+    {".mp3",  "audio/mpeg"},
+    {NULL,    NULL}
+};
+
 static const char *get_mime_type(const char *name)
 {
     char *ext = strrchr(name, '.');
     if (!ext) {
         return NULL;
     }
-
-    if (strcmp(ext, ".html") == 0 || strcmp(ext, ".htm") == 0) {
-        return "text/html";
-    }
-    if (strcmp(ext, ".jpg") == 0 || strcmp(ext, ".jpeg") == 0) {
-        return "image/jpeg";
-    }
-    if (strcmp(ext, ".gif") == 0) {
-        return "image/gif";
-    }
-    if (strcmp(ext, ".png") == 0) {
-        return "image/png";
-    }
-    if (strcmp(ext, ".css") == 0) {
-        return "text/css";
-    }
-    if (strcmp(ext, ".au") == 0) {
-        return "audio/basic";
-    }
-    if (strcmp(ext, ".wav") == 0) {
-        return "audio/wav";
-    }
-    if (strcmp(ext, ".avi") == 0) {
-        return "video/x-msvideo";
-    }
-    if (strcmp(ext, ".mpeg") == 0 ||  strcmp(ext, ".mpg") == 0) {
-        return "video/mpeg";
-    }
-    if (strcmp(ext, ".mp3") == 0) {
-        return "audio/mpeg";
+    
+    int i=0;
+    while (_mimeTypes[i].ext) {
+       if (strcmp(ext, _mimeTypes[i].ext) == 0) {
+           INFO2("[WS]: Mime type: %s", _mimeTypes[i].mime);
+           return _mimeTypes[i].mime;
+       }
+       i++;
     }
     return NULL;
 }
@@ -272,7 +274,7 @@ static void sendError(_WebClientConnection* cc, int status, char *title, char *e
 static void sendFile(_WebClientConnection* cc, char *path, struct stat *statbuf)
 {
     int fd = cc->connDescriptor;
-    int bytes_sent;
+    ssize_t bytes_sent;
 
     INFO2("[WS]: sendFile %s",path);
 
@@ -293,8 +295,12 @@ static void sendFile(_WebClientConnection* cc, char *path, struct stat *statbuf)
         int length = S_ISREG(statbuf->st_mode) ? statbuf->st_size : -1;
         sendHeaders(cc, 200, "OK", NULL, get_mime_type(path), length, statbuf->st_mtime);
 
+#ifdef __FreeBSD__
+        int err = sendfile(fdout, fd, 0, length, NULL, &bytes_sent, 0);
+#else
         bytes_sent = sendfile(fd,fdout,NULL,length);
-    
+        int err = 0;
+#endif
         /*while ((n = fread(data, 1, sizeof(data), file)) > 0) {
             //INFO2("read %d bytes from file",n);
 
@@ -305,7 +311,7 @@ static void sendFile(_WebClientConnection* cc, char *path, struct stat *statbuf)
             }
         }*/
     
-        if (length != bytes_sent) {
+        if (err || length != bytes_sent) {
             ERROR2("[WS]: Error on send file %s", path);
         }
 
