@@ -773,24 +773,13 @@ const char* lfCaption()
 // rely on previous strtok()
 static void setLfList(boolean_t useIcons) 
 {
-    char *token3 = strtok(NULL,",");
+    char *token3 = strtok(NULL,",\n");
     while (token3) {
-
-    	if (*token3 == '\n') {        // remove new-line at the start
-    	    token3++;
-    	}
-
-    	ListItem * item = malloc(sizeof(ListItem));
-
-    	int l = strlen(token3);
-    	if(*(token3 + l) == '\n') {	// remove new-line at the end
-    	    *(token3 + l) = '\0';
-    	}
-    	if(*(token3 + l - 1) == '\n') {  // shit happens
-    	    *(token3 + l - 1) = '\0';
-    	}
-
-    	if (strlen(token3) > 0) {   // avoid empty list item
+        
+        if (strlen(token3) > 0) {   // avoid empty list item
+            
+    	    ListItem * item = malloc(sizeof(ListItem));
+            
     	    char * semicolon = index(token3,':');
     	    if (useIcons && semicolon) {
     		    *semicolon = '\0';
@@ -803,8 +792,8 @@ static void setLfList(boolean_t useIcons)
     	    }
 
     	    _state.lf.items = listSingleAppend(_state.lf.items, item);
-    	}
-    	token3 = strtok(NULL,",");
+        }
+    	token3 = strtok(NULL,",\n");
     }
 }
 
