@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,15 +19,13 @@
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
 
+#include <errno.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <pwd.h>
-#include <errno.h>
+//#include <unistd.h>
 
 #include "common.h"
 #include "lib_wrapper.h"
@@ -113,16 +111,16 @@ void initLog()
         }
         if(getuid()==0 && conf.uid) { // do not open file as superuser
             #ifdef __cplusplus
-	    int dummy = 
-	    #endif
+	        int dummy = 
+	        #endif
 	        chown(logfile,conf.uid,conf.gid);
         }
 
         mutexNew(M_LOG);
         
-	printTime();
+	    printTime();
         sprintf(tmp, "anyRemote v%s", PACKAGE_VERSION);
-	logger(L_CFG, tmp);
+	    logger(L_CFG, tmp);
     }
 }
 
@@ -152,10 +150,10 @@ static void logger2(const char *head, const char *str)
     if (fplog!=NULL) {
         if (head && strcmp(head, "CFG") != 0) {
             
-	    time(&logtime);
+	        time(&logtime);
             struct tm* timeinfo = localtime (&logtime );
            
-	    //gettimeofday(&tv, &tz);
+	        //gettimeofday(&tv, &tz);
 
             char stime[32];
             stime[8] = '\0';
@@ -173,7 +171,7 @@ static void logger2(const char *head, const char *str)
         fclose(fplog);
     } else {
         printf("Can not open log file >%s<\n", logfile);
-	logfile[0] = '\0';  // print this error only once
+	    logfile[0] = '\0';  // print this error only once
     }
 
     mutexUnlock(M_LOG);
@@ -237,16 +235,17 @@ char* getCmdFile()
 
 static boolean_t isDataCacheable(int what) 
 {
-    if (what == ID_SET_BG     ||
-        what == ID_SET_FG     ||
-        what == ID_SET_FONT   ||
-        what == ID_SET_ICONS  ||
-        what == ID_SET_LAYOUT ||
-        what == ID_SET_STATUS ||
-        what == ID_SET_TITLE  ||
-        what == ID_SET_VOLUME ||
+    if (what == ID_SET_BG      ||
+        what == ID_SET_CAPTION ||
+        what == ID_SET_FG      ||
+        what == ID_SET_FONT    ||
+        what == ID_SET_ICONS   ||
+        what == ID_SET_LAYOUT  ||
+        what == ID_SET_STATUS  ||
+        what == ID_SET_TITLE   ||
+        what == ID_SET_VOLUME  ||
         what == ID_SET_COVER) {
-	return BOOL_YES;
+        return BOOL_YES;
     } else {
         return BOOL_NO;
     }
@@ -381,6 +380,8 @@ int flushData()
 
     flushTimers = getTimers();
     forgetTimers();
+    
+    freeCachedData();
 
     // Now we ready to load new cfg.
     return EXIT_OK;
@@ -401,13 +402,13 @@ void flushOldConf()
 
 void printHelp()
 {
-    printf("Usage: anyremote [-f conf.file] [-s connect string] [-log] [-a] \n");
+    printf("Usage: anyremote [-f conf.file] [-s peer[,peer ...]] [-log] [-a] \n");
     printf("                 [-fe port] [-u username] [-name SDP service name] [-password]\n");
     printf("   or: anyremote -h|-v\n\n");
     printf(" -h|--help  print this help\n");
     printf(" -v|--version  print version of anyRemote\n");
-    printf(" -f /path/to/file  specify configurational file\n");
-    printf(" -s connect string,  possible values\n");
+    printf(" -f /path/to/file  specify configuration file\n");
+    printf(" -s peer[,peer ...],  possible values\n");
     printf("    bluetooth:<channel> (Server mode - bluetooth connection)\n");
     printf("    socket:<port>       (Server mode - TCP/IP connection)\n");
     printf("    web:<port>          (Server mode - Web interface)\n");
@@ -420,9 +421,11 @@ void printHelp()
     printf("    /dev/ircomm#        (AT mode - IR connection)\n");
     printf("    ilirc:<AF_LOCAL socket file> (use with inputlircd)\n");
     printf("    stdin\n");
-    printf("    default value for connect string is bluetooth:19\n");
+    printf("  Default peer value is bluetooth:19\n");
+    printf("  It is possible to specify several peers for Server mode configuration files only\n");
+    printf("  It is possible to specify only single peer of web: or cmxml: type\n");
     printf(" -log print verbose logging information to $HOME/.anyRemote/anyremote.log\n");
-    printf(" -a	reconnect automatically in case of connection failure\n");
+    printf(" -a	reconnect automatically in case of connection failure, used only in AT-mode\n");
     printf(" -fe <port> work as backend for GUI frontend. Use specified port to connect to frontend.\n");
     printf(" -name <SDP service name> if bluetooth connection is used, allows one to specify SDP service name\n");
     printf(" -password ask password on connect\n");
@@ -472,6 +475,13 @@ void sendDisconnect()
     sendToMainThread(i);
 }
 
+void stripCommandEnding(char *s)
+{
+    if (s && strlen(s) >= 2) {
+        s[strlen(s)-2] = '\0';   // strip );
+    }
+}
+
 //////////////////////////////////////////////////////////////////////////////////
 
 void errnoDebug(const char* tag, int err) 
@@ -571,7 +581,7 @@ char * convCharset(char *str, size_t size, int direction)
         buf = (char *) malloc(bufsize);
         holder = buf;
 	
-	iconv_t cnv = (direction == CNV_TO ? convertorTo : convertorFrom);
+	    iconv_t cnv = (direction == CNV_TO ? convertorTo : convertorFrom);
 
         r = iconv(cnv, &str, &insize, &buf, &outsize);
         if (r < 0 || insize != 0) {
@@ -586,7 +596,7 @@ char * convCharset(char *str, size_t size, int direction)
         buf -= size;
         out = (char *) malloc(size + 1);
         memcpy(out, buf, size);
-	out[size] = '\0';
+	    out[size] = '\0';
 	
         free(holder);
     }
