@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,9 +22,6 @@
 #ifndef _UTILS_H
 #define _UTILS_H
 
-#include <sys/types.h>
-#include <pwd.h>
-
 #include "parse.h"
 
 #define LOGFILE 	"/anyremote.log"
@@ -80,6 +77,8 @@ int         needConvert     (void);
 
 void        errnoDebug(const char* tag, int err);
 
+void        stripCommandEnding(char *s);
+
 #ifdef USE_ICONV
 void        encodingHook     (void);
 void        closeConvertor   (void);
