@@ -135,10 +135,15 @@ static KeyCode keysymStr2keycode(Display* disp, const char* keysymStr)
 // Input can be something like:
 // key,a,key,by_value,40,keyup,b,keydown,c,mouse,1,mouseup,1,mousedown,1,mousemove,100,100,mousermove,10,10,mousedblclick,sleep,1000
 //
-int emulateCommands(char* cmdString)
+int emulateCommands(int subtype, const char *descr, const char* cmdString, cmdParams* params)
 {
-#ifdef USE_XTEST
-
+    logger(L_INF, "Command: Emulate");
+    
+    #ifdef USE_XTEST
+    if (descr == NULL) {
+        logger(L_ERR, "[EX]: emulateCommands(): no command");
+        return EXIT_NOK;
+    }
     if (disp == NULL) {
         disp = XOpenDisplay(NULL);
     }
@@ -146,7 +151,7 @@ int emulateCommands(char* cmdString)
         return EXIT_NOK;
     }
 
-    char * cmd = strdup(cmdString);
+    char * cmd = strdup(descr);
 
     KeyCode fakeKey;
 
@@ -289,7 +294,7 @@ int emulateCommands(char* cmdString)
     XFlush(disp);
 
     free(cmd);
-#endif
+    #endif
 
     return EXIT_OK;
 }
