@@ -22,7 +22,9 @@
 #ifndef _XEMULATE_H_
 #define _XEMULATE_H_
 
-int  emulateCommands(char* cmdString);
+#include "parse.h"
+
+int  emulateCommands(int subtype, const char *descr, const char* cmdString, cmdParams* params);
 void freeDisplay(void);
 
 #endif
